#pragma once



namespace DMH
{

	template<typename T>
	union tvec4 {
		T data[4];
		struct {
			T x;      /**< Alias for first component  */
			T y;      /**< Alias for second component */
			T z;      /**< Alias for third component  */
			T w;      /**< Alias for fourht component */
		};
		struct {
			T x_;     /**< Alias for first component  */
			T y_;     /**< Alias for second component */
			T width;  /**< Alias for third component  */
			T height; /**< Alias for fourth component */
		};
		struct {
			T r;      /**< Alias for first component  */
			T g;      /**< Alias for second component */
			T b;      /**< Alias for third component  */
			T a;      /**< Alias for fourth component */
		};
		struct {
			T red;    /**< Alias for first component  */
			T green;  /**< Alias for second component */
			T blue;   /**< Alias for third component  */
			T alpha;  /**< Alias for fourth component */
		};
		struct {
			T vstart; /**< Alias for first component  */
			T vcount; /**< Alias for second component */
			T istart; /**< Alias for third component  */
			T icount; /**< Alias for fourth component */
		};

		tvec4() {}
		tvec4(T x, T y, T z, T w) : x(x), y(y), z(z), w(w) {}
	};

	/**
	* Tuple of 3 elements.
	*
	* Each field can be addressed using several aliases:
	*  - First component:  <b>x</b>, <b>r</b> or <b>red</b>
	*  - Second component: <b>y</b>, <b>g</b> or <b>green</b>
	*  - Third component:  <b>z</b>, <b>b</b> or <b>blue</b>
	*
	*/
	template<typename T>
	union tvec3 {
		T data[3];    /**< All compoments at once     */
		struct {
			T x;      /**< Alias for first component  */
			T y;      /**< Alias for second component */
			T z;      /**< Alias for third component  */
		};
		struct {
			T r;      /**< Alias for first component  */
			T g;      /**< Alias for second component */
			T b;      /**< Alias for third component  */
		};
		struct {
			T red;    /**< Alias for first component  */
			T green;  /**< Alias for second component */
			T blue;   /**< Alias for third component  */
		};

		tvec3() {}
		tvec3(T x, T y, T z) : x(x), y(y), z(z) {}
	};

	/**
	* Tuple of 2 elements.
	*
	* Each field can be addressed using several aliases:
	*  - First component: <b>x</b>, <b>s</b> or <b>start</b>
	*  - Second component: <b>y</b>, <b>t</b> or <b>end</b>
	*
	*/
	template<typename T>
	union tvec2
	{
		T data[2];    /**< All compoments at once     */
		struct {
			T x;      /**< Alias for first component  */
			T y;      /**< Alias for second component */
		};
		struct {
			T s;      /**< Alias for first component  */
			T t;      /**< Alias for second component */
		};
		struct {
			T start;  /**< Alias for first component  */
			T end;    /**< Alias for second component */
		};

		tvec2() {}
		tvec2(T x, T y) : x(x), y(y) {}
	};

	typedef tvec4<int> ivec4;
	typedef tvec3<int> ivec3;
	typedef tvec2<int> ivec2;

	typedef tvec4<float> vec4;
	typedef tvec3<float> vec3;
	typedef tvec2<float> vec2;


	typedef tvec4<double> dvec4;
	typedef tvec3<double> dvec3;
	typedef tvec2<double> dvec2;

	typedef tvec4<long> lvec4;
	typedef tvec3<long> lvec3;
	typedef tvec2<long> lvec2;


	inline bool InRect(ivec4 r, int x, int y)
	{
		if (x >= r.x && x <= r.x + r.width)
		{
			if (y >= r.y && y <= r.y + r.height)
			{
				return true;
			}
		}
		return false;
	}

	inline bool equals(ivec4 l, ivec4 r)
	{

		if (l.x != r.x) return false;
		if (l.y != r.y) return false;
		if (l.width != r.width) return false;
		if (l.height != r.height) return false;

		return true;
	}

	//typedef tvec2<float> vec2;typedef tvec2<int> Point;






	struct Rect
	{
		Rect() : x(0), y(0), w(0), h(0) {}
		Rect(const ivec2 & p) : x(p.x), y(p.y), w(0), h(0) {}
		Rect(int ix, int iy, int iw = 0, int ih = 0) : x(ix), y(iy), w(iw), h(ih) {}
		Rect(const Rect & r) : x(r.x), y(r.y), w(r.w), h(r.h) {}

		const Rect& operator= (const Rect & r) { this->x = r.x; this->y = r.y; this->w = r.w; this->h = r.h; return *this; }
		bool isPtInside(int px, int py) { if ((px > (x + w)) || (px < x) || (py > (y + h)) || (py < y)) { return false; } return true; };
		bool isPtInside(ivec2 p) { if ((p.x > (x + w)) || (p.x < x) || (y > (p.y + h)) || (p.y < y)) { return false; } return true; };
		bool isPtInside(lvec2 p) { if ((p.x > (x + w)) || (p.x < x) || (y > (p.y + h)) || (p.y < y)) { return false; } return true; };
		int left() { return x; }
		int top() { return y; }
		int right() { return x + w; }
		int bottom() { return y + h; }


		int x, y;
		int w, h;

		static const Rect null;
	};

	struct Rectf
	{
		Rectf() : x(0), y(0), w(0), h(0) {}
		Rectf(const ivec2 & p) : x((float)p.x), y((float)p.y), w(0), h(0) {}
		Rectf(int ix, int iy, int iw = 0, int ih = 0) : x((float)ix), y((float)iy), w((float)iw), h((float)ih) {}
		Rectf(float ix, float iy, float iw = 0, float ih = 0) : x((float)ix), y((float)iy), w((float)iw), h((float)ih) {}
		Rectf(const Rectf & r) : x(r.x), y(r.y), w(r.w), h(r.h) {}
		Rectf(const Rect & r) : x((float)r.x), y((float)r.y), w((float)r.w), h((float)r.h) {}

		float left() { return x; }
		float top() { return y; }
		float right() { return x + w; }
		float bottom() { return y + h; }

		bool isPtInside(float px, float py) { if ((px > (x + w)) || (px < x) || (py > (y + h)) || (py < y)) { return false; } return true; };
		bool isPtInside(vec2 p) { if ((p.x > (x + w)) || (p.x < x) || (y > (p.y + h)) || (p.y < y)) { return false; } return true; };

		const Rectf& operator= (const Rectf & r) { this->x = r.x; this->y = r.y; this->w = r.w; this->h = r.h; return *this; }
		const Rectf& operator= (const Rect & r) { this->x = (float)r.x; this->y = (float)r.y; this->w = (float)r.w; this->h = (float)r.h; return *this; }

		float x, y;
		float w, h;

		static const Rectf null;
	};

};